/* Copyright 1996, Jan D. Wolter and Steven R. Weiss, All Rights Reserved. */

/* Functions to generate http headers */

#include "backtalk.h"

#if TM_IN_SYS_TIME
#include <sys/time.h>
#else
#include <time.h>
#endif /* TM_IN_SYS_TIME */

#include "sysdict.h"
#include "http.h"
#include "stack.h"
#include "str.h"
#include "cgi_cookie.h"

/* Linked list of cookies to be set in the current page */

struct cook {
	char *name;		/* Name of cookie */
	char *value;		/* Value of cookie */
	long expire;		/* Expiration time of cookie - 0= none */
	struct cook *next;	/* Next cookie to set */
	} *cookies= NULL;
int http_sent= FALSE;		/* Have we sent the http headers? */

extern FILE *output;


/* HTTP_TIME - Convert a Unix time integer to a string in the http format:
 *    Wdy, DD-Mon-YYYY HH:MM:SS GMT
 * The result is stored in the given buffer, which should always be at
 * least 30 bytes long.
 */

char *http_time(time_t time, char *bf)
{
    struct tm *gmt;
#ifndef HAVE_STRFTIME
    static char *wkd= "SunMonTueWedThuFriSat";
    static char *mon= "JanFebMarAprMayJunJulAugSepOctNovDec";
#endif /* !HAVE_STRFTIME */

    gmt= gmtime(&time);
#ifdef HAVE_STRFTIME
    strftime(bf, 30, "%a, %d-%b-%Y %H:%M:%S GMT",gmt);
#else
    sprintf(bf, "%3.3s, %02d-%3.3s-%04d %02d:%02d:%02d GMT",
	wkd[gmt->tm_wday*3],
	gmt->tm_mday, mon[gmt->tm_mon*3], gmt->tm_year+1900,
	gmt->tm_hour, gmt->tm_min, gmt->tm_sec);
#endif /* HAVE_STRFTIME */
    return bf;
}


/* SET_COOKIE - Add a cookie to the list of cookies to be set by the
 * current page.  If the expiration date is zero, the cookie expires when
 * the user exits his browser.  "name" and "value" should be in malloc'ed
 * memory, which we will eventually be freeing.  The cookie value should not
 * contain spaces or semicolons.  If it is too late to add a cookie (we have
 * already printed something), then return nonzero.
 */

int set_cookie(char *name, char *value, time_t expire)
{
    struct cook *nck,*ck,*pck;

    if (http_sent) return 1;

    /* Walk through the list of cookies, looking for duplicates */
    for (ck= cookies, pck= NULL; ck != NULL; pck= ck, ck= ck->next)
	if (!strcmp(name,ck->name))
	{
	    /* Overwrite an existing cookie */
	    free(name);
	    free(ck->value);
	    ck->value= value;
	    ck->expire= expire;
	    return 0;
	}

    /* Make a new cookie list entry */
    nck= (struct cook *)malloc(sizeof(struct cook));
    nck->name= name;
    nck->value= value;
    nck->expire= expire;
    nck->next= NULL;

    /* Append the new cookie to the list */
    if (pck == NULL)
	cookies= nck;
    else
	pck->next= nck;

    return 0;
}


/* FUNC_SETCOOKIE:   <name> <value> <expires> setcookie -
 * Set a cookie.  The value should contain no spaces or semicolons.  Expires
 * is a unix time, or zero if the cookie should expire when the browser exits.
 */

void func_setcookie()
{
    time_t expire= pop_time();
    char *value= pop_string();
    char *name= pop_string();

    set_cookie(name, value, expire);
}


/* SEND_HTTP_HEADERS - Output the http headers for the current page based
 * on the current system dictionary settings.  If they have already been
 * sent, do nothing.
 */

void send_http_headers()
{
    struct cook *ck, *nck;

    /* If headers already sent, or have been turned off, do nothing */
    if (http_sent || !bdict(VAR_HTTP_HEADERS))
	return;

    http_sent= TRUE;

    /* Always output a content type header */
    fprintf(output,"Content-Type: %s\n",sdict(VAR_HTTP_CONTENT_TYPE));

    /* If caching has been turned off, do various things to suppress it */
    if (bdict(VAR_HTTP_NO_CACHE))
    {
	fprintf(output,"Expires:  Thu, 01 Jan 1970 00:00:00 GMT\n");
	fprintf(output,"Pragma:  no-cache\n");
	fprintf(output,"Cache-Control: no-cache\n");
    }
    /* Otherwise, if an expiration time has been set, send that */
    else if (tdict(VAR_HTTP_EXPIRES) != 0)
    {
	char bf[30];
	fprintf(output,"Expires: %s\n", http_time(tdict(VAR_HTTP_EXPIRES),bf));
    }

    /* Set our cookies */
    for (ck= cookies; ck != NULL; ck= nck)
    {
	fprintf(output, "Set-Cookie: %s=%s; path=/", ck->name, ck->value);
	if (ck->expire > 0L)
	{
	    char bf[30];
	    fprintf(output,"; expires=%s",http_time(ck->expire,bf));
	}
	putc('\n',output);

	nck= ck->next;
	free(ck->name);
	free(ck->value);
	free(ck);
    }
    cookies= NULL;

    /* If an redirect has been set, send that */
    if (*sdict(VAR_HTTP_LOCATION) != '\0')
	fprintf(output,"Location: %s\n", sdict(VAR_HTTP_LOCATION));

    /* End the http headers */
    putc('\n',output);
}


/* FREE_COOKIES - discard any cookies still sitting in memory.  This is for
 * debugging memory management only.
 */

#ifdef CLEANUP
void free_cookies()
{
struct cook *ck, *nck;

    for (ck= cookies; ck != NULL; ck= nck)
    {
	nck= ck->next;
	free(ck->name);
	free(ck->value);
	free(ck);
    }
    cookies= NULL;
}
#endif /*CLEANUP*/


/* FUNC_GETCOOKIE:   <name> getcookie <value> 0
 *                   <name> getcookie 1
 * 
 * Look up the value of the named cookie.  If the cookie is not defined, just
 * push a one.  Otherwise, push its value and a zero.
 */

void func_getcookie()
{
char *name= pop_string();
char *value= get_cookie(name);

    free(name);
    if (value != NULL)
    {
	push_string(value,FALSE);
	push_integer(0);
    }
    else
	push_integer(1);
}
